/**

 * File Name:   main.c
 * 
 * Summary: This is the main file to initialize the RN4020 module 
 *          on the Curiosity panel
 * 
 * MPLAB             :  MPLAB X v2.35 or v3.00
 * 
 * D4-D6 : LEDs, mainly for status feedback
 * 
 * BLE_  : This is the RN4020 unit with the following pins:
 *            WKHW: Hardware wake up pin, connected to RC0 by default on the 
 *                  Curiosity board. It is now mandatory to use this pin, but
 *                  the device is able to work without setting the pin high
 *                  if the firmware version not higher than 1.2
 *            WKSW: Software wake up pin, connected to RC1. Setting this pin to 
 *                  high starts the RN4020
 *            TX:   RB7
 *            RX:   RB5
 *         No other pins are required for the initialization, however the PIO 
 *         pins can be used for onboard status checks.
 * 
 * OLED_ : I have an OLED C click from MikroElektronika, used in this project 
 *         as a character "terminal" to display feedbacks from the RN4020
 *         If you have other options in your PIC to display the received data
 *         replace these lines with your solution.
 * 
 * S1    : Configured as a two state button. In pressed state decrements the 
 *         <batt> variable to simulate battery service. If a receiver is 
 *         connected via bluetooth service it is able to receive the battery 
 *         value in hexadecimal format.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#include "mcc_generated_files/mcc.h"
#include "main.h"
#include "oled_c.h"
#include "rn4020.h"

const unsigned char hexs[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

volatile int mx = 0;
volatile int my = 0;
/*
                         Main application
 */
void main(void) {
    // initialize the device
    SYSTEM_Initialize();
    
    // When using interrupts, you need to set the Global and Peripheral Interrupt Enable bits
    // Use the following macros to:

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();
    
    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();
    
    WKHW_SetHigh();
    
    RW_SetLow();
    
    OLED_C_Init();
    OLED_C_Background();
    
    WKSW_SetHigh();
    
    //BLE_ToggleEcho();
    
    BLE_FullReset();
    waitForData();
    
    delay_ms(4000);
    
    unsigned char service[9] = "C0000000";
    unsigned char role[9] = "00000000";
    
    BLE_SetService(service);
    
    BLE_SetRole(role);
    
    BLE_Reboot();
    waitForData();
    delay_ms(2000);
    
    BLE_GetVersion();
    waitForData();
    //BLE_GetInfo();
    
    BLE_GetServerServices();
    waitForData();
    
    BLE_StartBroadcast();
    waitForData();
    
    uint8_t batt = 100;
    
    unsigned char btn = 0;
    
    D4_SetHigh();
    
    while (1){
        
        if (EUSART_DataReady){
            D6_SetHigh();
            writeData();
            D6_SetLow();
        }
        
        if (btn == 0){
            if (S1_PORT == 0){
                btn = 1;
                batt--;
                if (batt < 1){
                    batt = 100;
                }
                unsigned char blev[2];
                uint8_t bh = batt >> 4;
                uint8_t bl = batt & 0x0F;
                blev[0] = dec2hex(bh);
                blev[1] = dec2hex(bl);
                BLE_SendBatteryValue(blev);
            }
        } else {
            if (S1_PORT == 1){
                btn = 0;
            }
        }
    }
}

unsigned char dec2hex(uint8_t dec){
    return hexs[dec];
}

void waitForData(){
    while (!EUSART_DataReady){
    }
    writeData();
}

void writeData(){
    D5_SetLow();
    while (eusartRxCount > 0){
        uint8_t in = EUSART_Read();
        if ((in != 13) && (in != 10)){
            OLED_C_print_char(mx, my, in);
            my = my + 6;
            if (my > 91){
                my = 0;
                mx = mx + 6;
                if (mx > 91) {
                    OLED_C_Background();
                    mx = 0;
                }
            }
        } else {
            if (in == 10){
                my = 0;
                mx = mx + 8;
                if (mx > 91){
                    OLED_C_Background();
                    mx = 0;
                }
            }
        }
    }
}


/*
void delay_ms(int msec){
    TMR1_StartTimer();
    for (int x = 0; x < msec; x++){
        while(!TMR1IF);
        TMR1_Reload();
        TMR1IF = 0;
    }
    //TMR1_StopTimer();
}
*/


/**
 End of File
 */